@@ -14,84 +14,82 @@
 ## with this program; if not, write to the Free Software Foundation, Inc.,
 ## 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
-# Base low-level functions used by the API proper.
-# Unlikely to be used directly unless you're expanding the API.
+"""Base low-level functions as API for upper layers."""
 
 from __future__ import annotations
 
+import dataclasses
 import logging
-import threading as _threading
+import platform
+import struct
+import threading
+import typing
 
-from collections import namedtuple
 from contextlib import contextmanager
-from random import getrandbits as _random_bits
-from struct import pack as _pack
-from time import time as _timestamp
-
-import hidapi as _hid
+from random import getrandbits
+from time import time
+from typing import Any
+from typing import Callable
 
+from . import base_usb
+from . import common
+from . import descriptors
 from . import exceptions
-from . import hidpp10_constants as _hidpp10_constants
-from . import hidpp20
-from . import hidpp20_constants as _hidpp20_constants
-from .base_usb import ALL as _RECEIVER_USB_IDS
-from .common import strhex as _strhex
-from .descriptors import DEVICES as _DEVICES
+from .common import LOGITECH_VENDOR_ID
+from .common import BusID
+from .hidpp10_constants import ErrorCode as Hidpp10ErrorCode
+from .hidpp20_constants import ErrorCode as Hidpp20ErrorCode
+
+if typing.TYPE_CHECKING:
+    import gi
+
+    from hidapi.common import DeviceInfo
+
+    gi.require_version("Gdk", "3.0")
+    from gi.repository import GLib  # NOQA: E402
+
+if platform.system() == "Linux":
+    import hidapi.udev_impl as hidapi
+else:
+    import hidapi.hidapi_impl as hidapi
 
 logger = logging.getLogger(__name__)
 
-_hidpp20 = hidpp20.Hidpp20()
 
-#
-#
-#
+class HIDAPI(typing.Protocol):
+    def find_paired_node_wpid(self, receiver_path: str, index: int):
+        ...
+
+    def find_paired_node(self, receiver_path: str, index: int, timeout: int):
+        ...
+
+    def open(self, vendor_id, product_id, serial=None):
+        ...
+
+    def open_path(self, path):
+        ...
+
+    def enumerate(self, filter_func: Callable[[int, int, int, bool, bool], dict[str, typing.Any]]) -> DeviceInfo:
+        ...
+
+    def monitor_glib(
+        self, glib: GLib, callback: Callable, filter_func: Callable[[int, int, int, bool, bool], dict[str, typing.Any]]
+    ) -> None:
+        ...
+
+    def read(self, device_handle, bytes_count, timeout_ms):
+        ...
+
+    def write(self, device_handle: int, data: bytes) -> int:
+        ...
+
+    def close(self, device_handle) -> None:
+        ...
 
 
-def _wired_device(product_id, interface):
-    return {"vendor_id": 1133, "product_id": product_id, "bus_id": 3, "usb_interface": interface, "isDevice": True}
+hidapi = typing.cast(HIDAPI, hidapi)
 
-
-def _bt_device(product_id):
-    return {"vendor_id": 1133, "product_id": product_id, "bus_id": 5, "isDevice": True}
-
-
-DEVICE_IDS = []
-
-for _ignore, d in _DEVICES.items():
-    if d.usbid:
-        DEVICE_IDS.append(_wired_device(d.usbid, d.interface if d.interface else 2))
-    if d.btid:
-        DEVICE_IDS.append(_bt_device(d.btid))
-
-
-def other_device_check(bus_id, vendor_id, product_id):
-    """Check whether product is a Logitech USB-connected or Bluetooth device based on bus, vendor, and product IDs
-    This allows Solaar to support receiverless HID++ 2.0 devices that it knows nothing about"""
-    if vendor_id != 0x46D:  # Logitech
-        return
-    if bus_id == 0x3:  # USB
-        if product_id >= 0xC07D and product_id <= 0xC094 or product_id >= 0xC32B and product_id <= 0xC344:
-            return _wired_device(product_id, 2)
-    elif bus_id == 0x5:  # Bluetooth
-        if product_id >= 0xB012 and product_id <= 0xB0FF or product_id >= 0xB317 and product_id <= 0xB3FF:
-            return _bt_device(product_id)
-
-
-def product_information(usb_id: int | str) -> dict:
-    if isinstance(usb_id, str):
-        usb_id = int(usb_id, 16)
-
-    for r in _RECEIVER_USB_IDS:
-        if usb_id == r.get("product_id"):
-            return r
-    return {}
-
-
-#
-#
-#
-
-_SHORT_MESSAGE_SIZE = 7
+SHORT_MESSAGE_SIZE = 7
 _LONG_MESSAGE_SIZE = 20
 _MEDIUM_MESSAGE_SIZE = 15
 _MAX_READ_SIZE = 32
@@ -100,13 +98,7 @@ HIDPP_SHORT_MESSAGE_ID = 0x10
 HIDPP_LONG_MESSAGE_ID = 0x11
 DJ_MESSAGE_ID = 0x20
 
-# mapping from report_id to message length
-report_lengths = {
-    HIDPP_SHORT_MESSAGE_ID: _SHORT_MESSAGE_SIZE,
-    HIDPP_LONG_MESSAGE_ID: _LONG_MESSAGE_SIZE,
-    DJ_MESSAGE_ID: _MEDIUM_MESSAGE_SIZE,
-    0x21: _MAX_READ_SIZE,
-}
+
 """Default timeout on read (in seconds)."""
 DEFAULT_TIMEOUT = 4
 # the receiver itself should reply very fast, within 500ms
@@ -116,12 +108,64 @@ _DEVICE_REQUEST_TIMEOUT = DEFAULT_TIMEOUT
 # when pinging, be extra patient (no longer)
 _PING_TIMEOUT = DEFAULT_TIMEOUT
 
-#
-#
-#
+
+@dataclasses.dataclass
+class HIDPPNotification:
+    report_id: int
+    devnumber: int
+    sub_id: int
+    address: int
+    data: bytes
+
+    def __str__(self):
+        text_as_hex = common.strhex(self.data)
+        return f"Notification({self.report_id:02x},{self.devnumber},{self.sub_id:02X},{self.address:02X},{text_as_hex})"
 
 
-def match(record, bus_id, vendor_id, product_id):
+def _usb_device(product_id: int, usb_interface: int) -> dict[str, Any]:
+    return {
+        "vendor_id": LOGITECH_VENDOR_ID,
+        "product_id": product_id,
+        "bus_id": BusID.USB,
+        "usb_interface": usb_interface,
+        "isDevice": True,
+    }
+
+
+def _bluetooth_device(product_id: int) -> dict[str, Any]:
+    return {"vendor_id": LOGITECH_VENDOR_ID, "product_id": product_id, "bus_id": BusID.BLUETOOTH, "isDevice": True}
+
+
+KNOWN_DEVICE_IDS = []
+
+for _ignore, d in descriptors.DEVICES.items():
+    if d.usbid:
+        usb_interface = d.interface if d.interface else 2
+        KNOWN_DEVICE_IDS.append(_usb_device(d.usbid, usb_interface))
+    if d.btid:
+        KNOWN_DEVICE_IDS.append(_bluetooth_device(d.btid))
+
+
+def _other_device_check(bus_id: int, vendor_id: int, product_id: int) -> dict[str, Any] | None:
+    """Check whether product is a Logitech USB-connected or Bluetooth device based on bus, vendor, and product IDs
+    This allows Solaar to support receiverless HID++ 2.0 devices that it knows nothing about"""
+    if vendor_id != LOGITECH_VENDOR_ID:
+        return
+
+    device_info = None
+    if bus_id == BusID.USB and (0xC07D <= product_id <= 0xC094 or 0xC32B <= product_id <= 0xC344):
+        device_info = _usb_device(product_id, 2)
+    elif bus_id == BusID.BLUETOOTH and (0xB012 <= product_id <= 0xB0FF or 0xB317 <= product_id <= 0xB3FF):
+        device_info = _bluetooth_device(product_id)
+    return device_info
+
+
+def product_information(usb_id: int) -> dict[str, Any]:
+    """Returns hardcoded information from USB receiver."""
+    return base_usb.get_receiver_info(usb_id)
+
+
+def _match(record: dict[str, Any], bus_id: int, vendor_id: int, product_id: int):
     return (
         (record.get("bus_id") is None or record.get("bus_id") == bus_id)
         and (record.get("vendor_id") is None or record.get("vendor_id") == vendor_id)
@@ -129,47 +173,64 @@ def match(record, bus_id, vendor_id, product_id):
     )
 
 
-def filter_receivers(bus_id, vendor_id, product_id, hidpp_short=False, hidpp_long=False):
-    """Check that this product is a Logitech receiver and if so return the receiver record for further checking"""
-    for record in _RECEIVER_USB_IDS:  # known receivers
-        if match(record, bus_id, vendor_id, product_id):
+def _filter_receivers(
+    bus_id: int, vendor_id: int, product_id: int, _hidpp_short: bool = False, _hidpp_long: bool = False
+) -> dict[str, Any]:
+    """Check that this product is a Logitech receiver and return it.
+
+    Filters based on bus_id, vendor_id and product_id.
+
+    If so return the receiver record for further checking.
+    """
+    try:
+        record = base_usb.get_receiver_info(product_id)
+        if _match(record, bus_id, vendor_id, product_id):
             return record
-    if vendor_id == 0x046D and 0xC500 <= product_id <= 0xC5FF:  # unknown receiver
+    except ValueError:
+        pass
+
+    if vendor_id == LOGITECH_VENDOR_ID and 0xC500 <= product_id <= 0xC5FF:  # unknown receiver
         return {"vendor_id": vendor_id, "product_id": product_id, "bus_id": bus_id, "isDevice": False}
+    return None
 
 
 def receivers():
     """Enumerate all the receivers attached to the machine."""
-    yield from _hid.enumerate(filter_receivers)
+    yield from hidapi.enumerate(_filter_receivers)
 
 
-def filter(bus_id, vendor_id, product_id, hidpp_short=False, hidpp_long=False):
+def _filter_products_of_interest(
+    bus_id: int, vendor_id: int, product_id: int, hidpp_short: bool = False, hidpp_long: bool = False
+) -> dict[str, Any] | None:
     """Check that this product is of interest and if so return the device record for further checking"""
-    record = filter_receivers(bus_id, vendor_id, product_id, hidpp_short, hidpp_long)
+    record = _filter_receivers(bus_id, vendor_id, product_id, hidpp_short, hidpp_long)
     if record:  # known or unknown receiver
         return record
-    for record in DEVICE_IDS:  # known devices
-        if match(record, bus_id, vendor_id, product_id):
+
+    for record in KNOWN_DEVICE_IDS:
+        if _match(record, bus_id, vendor_id, product_id):
             return record
     if hidpp_short or hidpp_long:  # unknown devices that use HID++
         return {"vendor_id": vendor_id, "product_id": product_id, "bus_id": bus_id, "isDevice": True}
     elif hidpp_short is None and hidpp_long is None:  # unknown devices in correct range of IDs
-        return other_device_check(bus_id, vendor_id, product_id)
+        return _other_device_check(bus_id, vendor_id, product_id)
+    return None
 
 
 def receivers_and_devices():
     """Enumerate all the receivers and devices directly attached to the machine."""
-    yield from _hid.enumerate(filter)
+    yield from hidapi.enumerate(_filter_products_of_interest)
 
 
-def notify_on_receivers_glib(callback):
-    """Watch for matching devices and notifies the callback on the GLib thread."""
-    return _hid.monitor_glib(callback, filter)
+def notify_on_receivers_glib(glib: GLib, callback: Callable):
+    """Watch for matching devices and notifies the callback on the GLib thread.
 
-
-#
-#
-#
+    Parameters
+    ----------
+    glib
+        GLib instance.
+    """
+    return hidapi.monitor_glib(glib, callback, _filter_products_of_interest)
 
 
 def open_path(path):
@@ -185,7 +246,7 @@ def open_path(path):
     :returns: an open receiver handle if this is the right Linux device, or
     ``None``.
     """
-    return _hid.open_path(path)
+    return hidapi.open_path(path)
 
 
 def open():
@@ -204,13 +265,11 @@ def close(handle):
     if handle:
         try:
             if isinstance(handle, int):
-                _hid.close(handle)
+                hidapi.close(handle)
             else:
                 handle.close()
-            # logger.info("closed receiver handle %r", handle)
             return True
         except Exception:
-            # logger.exception("closing receiver handle %r", handle)
             pass
 
     return False
@@ -233,15 +292,22 @@ def write(handle, devnumber, data, long_message=False):
     assert data is not None
     assert isinstance(data, bytes), (repr(data), type(data))
 
-    if long_message or len(data) > _SHORT_MESSAGE_SIZE - 2 or data[:1] == b"\x82":
-        wdata = _pack("!BB18s", HIDPP_LONG_MESSAGE_ID, devnumber, data)
+    if long_message or len(data) > SHORT_MESSAGE_SIZE - 2 or data[:1] == b"\x82":
+        wdata = struct.pack("!BB18s", HIDPP_LONG_MESSAGE_ID, devnumber, data)
     else:
-        wdata = _pack("!BB5s", HIDPP_SHORT_MESSAGE_ID, devnumber, data)
+        wdata = struct.pack("!BB5s", HIDPP_SHORT_MESSAGE_ID, devnumber, data)
     if logger.isEnabledFor(logging.DEBUG):
-        logger.debug("(%s) <= w[%02X %02X %s %s]", handle, ord(wdata[:1]), devnumber, _strhex(wdata[2:4]), _strhex(wdata[4:]))
+        logger.debug(
+            "(%s) <= w[%02X %02X %s %s]",
+            handle,
+            ord(wdata[:1]),
+            devnumber,
+            common.strhex(wdata[2:4]),
+            common.strhex(wdata[4:]),
+        )
 
     try:
-        _hid.write(int(handle), wdata)
+        hidapi.write(int(handle), wdata)
     except Exception as reason:
         logger.error("write failed, assuming handle %r no longer available", handle)
         close(handle)
@@ -266,15 +332,27 @@ def read(handle, timeout=DEFAULT_TIMEOUT):
         return reply
 
 
-# sanity checks on  message report id and size
-def check_message(data):
+def _is_relevant_message(data: bytes) -> bool:
+    """Checks if given id is a HID++ or DJ message.
+
+    Applies sanity checks on message report ID and message size.
+    """
     assert isinstance(data, bytes), (repr(data), type(data))
+
+    # mapping from report_id to message length
+    report_lengths = {
+        HIDPP_SHORT_MESSAGE_ID: SHORT_MESSAGE_SIZE,
+        HIDPP_LONG_MESSAGE_ID: _LONG_MESSAGE_SIZE,
+        DJ_MESSAGE_ID: _MEDIUM_MESSAGE_SIZE,
+        0x21: _MAX_READ_SIZE,
+    }
+
     report_id = ord(data[:1])
-    if report_id in report_lengths:  # is this an HID++ or DJ message?
+    if report_id in report_lengths:
         if report_lengths.get(report_id) == len(data):
             return True
         else:
-            logger.warning(f"unexpected message size: report_id {report_id:02X} message {_strhex(data)}")
+            logger.warning(f"unexpected message size: report_id {report_id:02X} message {common.strhex(data)}")
     return False
 
 
@@ -290,29 +368,31 @@ def _read(handle, timeout):
     try:
         # convert timeout to milliseconds, the hidapi expects it
         timeout = int(timeout * 1000)
-        data = _hid.read(int(handle), _MAX_READ_SIZE, timeout)
+        data = hidapi.read(int(handle), _MAX_READ_SIZE, timeout)
     except Exception as reason:
         logger.warning("read failed, assuming handle %r no longer available", handle)
         close(handle)
         raise exceptions.NoReceiver(reason=reason) from reason
 
-    if data and check_message(data):  # ignore messages that fail check
+    if data and _is_relevant_message(data):  # ignore messages that fail check
         report_id = ord(data[:1])
         devnumber = ord(data[1:2])
 
         if logger.isEnabledFor(logging.DEBUG) and (
             report_id != DJ_MESSAGE_ID or ord(data[2:3]) > 0x10
         ):  # ignore DJ input messages
-            logger.debug("(%s) => r[%02X %02X %s %s]", handle, report_id, devnumber, _strhex(data[2:4]), _strhex(data[4:]))
+            logger.debug(
+                "(%s) => r[%02X %02X %s %s]",
+                handle,
+                report_id,
+                devnumber,
+                common.strhex(data[2:4]),
+                common.strhex(data[4:]),
+            )
 
         return report_id, devnumber, data[2:]
 
 
-#
-#
-#
-
-
 def _skip_incoming(handle, ihandle, notifications_hook):
     """Read anything already in the input buffer.
 
@@ -322,14 +402,14 @@ def _skip_incoming(handle, ihandle, notifications_hook):
     while True:
         try:
             # read whatever is already in the buffer, if any
-            data = _hid.read(ihandle, _MAX_READ_SIZE, 0)
+            data = hidapi.read(ihandle, _MAX_READ_SIZE, 0)
         except Exception as reason:
             logger.error("read failed, assuming receiver %s no longer available", handle)
             close(handle)
             raise exceptions.NoReceiver(reason=reason) from reason
 
         if data:
-            if check_message(data):  # only process messages that pass check
+            if _is_relevant_message(data):  # only process messages that pass check
                 # report_id = ord(data[:1])
                 if notifications_hook:
                     n = make_notification(ord(data[:1]), ord(data[1:2]), data[2:])
@@ -340,23 +420,23 @@ def _skip_incoming(handle, ihandle, notifications_hook):
             return
 
 
-def make_notification(report_id, devnumber, data):
+def make_notification(report_id: int, devnumber: int, data: bytes) -> HIDPPNotification | None:
     """Guess if this is a notification (and not just a request reply), and
-    return a Notification tuple if it is."""
+    return a Notification if it is."""
 
     sub_id = ord(data[:1])
     if sub_id & 0x80 == 0x80:
         # this is either a HID++1.0 register r/w, or an error reply
-        return
+        return None
 
     # DJ input records are not notifications
     if report_id == DJ_MESSAGE_ID and (sub_id < 0x10):
-        return
+        return None
 
     address = ord(data[1:2])
     if sub_id == 0x00 and (address & 0x0F == 0x00):
         # this is a no-op notification - don't do anything with it
-        return
+        return None
 
     if (
         # standard HID++ 1.0 notification, SubId may be 0x40 - 0x7F
@@ -371,23 +451,11 @@ def make_notification(report_id, devnumber, data):
         # HID++ 2.0 feature notifications have the SoftwareID 0
         (address & 0x0F == 0x00)
     ):  # noqa: E129
-        return _HIDPP_Notification(report_id, devnumber, sub_id, address, data[2:])
+        return HIDPPNotification(report_id, devnumber, sub_id, address, data[2:])
+    return None
 
 
-_HIDPP_Notification = namedtuple("_HIDPP_Notification", ("report_id", "devnumber", "sub_id", "address", "data"))
-_HIDPP_Notification.__str__ = lambda self: "Notification(%02x,%d,%02X,%02X,%s)" % (
-    self.report_id,
-    self.devnumber,
-    self.sub_id,
-    self.address,
-    _strhex(self.data),
-)
-
-#
-#
-#
-
-request_lock = _threading.Lock()  # serialize all requests
+request_lock = threading.Lock()  # serialize all requests
 handles_lock = {}
 
 
@@ -396,7 +464,7 @@ def handle_lock(handle):
         if handles_lock.get(handle) is None:
             if logger.isEnabledFor(logging.INFO):
                 logger.info("New lock %s", repr(handle))
-            handles_lock[handle] = _threading.Lock()  # Serialize requests on the handle
+            handles_lock[handle] = threading.Lock()  # Serialize requests on the handle
     return handles_lock[handle]
 
 
@@ -413,8 +481,46 @@ def acquire_timeout(lock, handle, timeout):
             lock.release()
 
 
+def _get_next_sw_id() -> int:
+    """Returns 'random' software ID to separate replies from different devices.
+
+    Cycle the HID++ 2.0 software ID from 0x2 to 0xF to separate
+    results and notifications.
+    """
+    if not hasattr(_get_next_sw_id, "software_id"):
+        _get_next_sw_id.software_id = 0xF
+
+    if _get_next_sw_id.software_id < 0xF:
+        _get_next_sw_id.software_id += 1
+    else:
+        _get_next_sw_id.software_id = 2
+    return _get_next_sw_id.software_id
+
+
+def find_paired_node(receiver_path: str, index: int, timeout: int):
+    """Find the node of a device paired with a receiver."""
+    return hidapi.find_paired_node(receiver_path, index, timeout)
+
+
+def find_paired_node_wpid(receiver_path: str, index: int):
+    """Find the node of a device paired with a receiver.
+
+    Get wpid from udev.
+    """
+    return hidapi.find_paired_node_wpid(receiver_path, index)
+
+
 # a very few requests (e.g., host switching) do not expect a reply, but use no_reply=True with extreme caution
-def request(handle, devnumber, request_id, *params, no_reply=False, return_error=False, long_message=False, protocol=1.0):
+def request(
+    handle,
+    devnumber,
+    request_id: int,
+    *params,
+    no_reply: bool = False,
+    return_error: bool = False,
+    long_message: bool = False,
+    protocol: float = 1.0,
+):
     """Makes a feature call to a device and waits for a matching reply.
     :param handle: an open UR handle.
     :param devnumber: attached device number.
@@ -422,19 +528,14 @@ def request(handle, devnumber, request_id, *params, no_reply=False, return_error
     :param params: parameters for the feature call, 3 to 16 bytes.
     :returns: the reply data, or ``None`` if some error occurred. or no reply expected
     """
-
-    # import inspect as _inspect
-    # print ('\n  '.join(str(s) for s in _inspect.stack()))
-
     with acquire_timeout(handle_lock(handle), handle, 10.0):
         assert isinstance(request_id, int)
         if (devnumber != 0xFF or protocol >= 2.0) and request_id < 0x8000:
-            # For HID++ 2.0 feature requests, randomize the SoftwareId to make it
-            # easier to recognize the reply for this request. also, always set the
-            # most significant bit (8) in SoftwareId, to make notifications easier
-            # to distinguish from request replies.
+            # Always set the most significant bit (8) in SoftwareId,
+            # to make notifications easier to distinguish from request replies.
             # This only applies to peripheral requests, ofc.
-            request_id = (request_id & 0xFFF0) | 0x08 | _random_bits(3)
+            sw_id = _get_next_sw_id()
+            request_id = (request_id & 0xFFF0) | sw_id  # was 0x08 | getrandbits(3)
 
         timeout = _RECEIVER_REQUEST_TIMEOUT if devnumber == 0xFF else _DEVICE_REQUEST_TIMEOUT
         # be extra patient on long register read
@@ -442,12 +543,10 @@ def request(handle, devnumber, request_id, *params, no_reply=False, return_error
             timeout *= 2
 
         if params:
-            params = b"".join(_pack("B", p) if isinstance(p, int) else p for p in params)
+            params = b"".join(struct.pack("B", p) if isinstance(p, int) else p for p in params)
         else:
             params = b""
-        # if logger.isEnabledFor(logging.DEBUG):
-        #     logger.debug("(%s) device %d request_id {%04X} params [%s]", handle, devnumber, request_id, _strhex(params))
-        request_data = _pack("!H", request_id) + params
+        request_data = struct.pack("!H", request_id) + params
 
         ihandle = int(handle)
         notifications_hook = getattr(handle, "notifications_hook", None)
@@ -462,18 +561,17 @@ def request(handle, devnumber, request_id, *params, no_reply=False, return_error
             return None
 
         # we consider timeout from this point
-        request_started = _timestamp()
+        request_started = time()
         delta = 0
 
         while delta < timeout:
             reply = _read(handle, timeout)
-
             if reply:
                 report_id, reply_devnumber, reply_data = reply
                 if reply_devnumber == devnumber or reply_devnumber == devnumber ^ 0xFF:  # BT device returning 0x00
                     if (
                         report_id == HIDPP_SHORT_MESSAGE_ID
-                        and reply_data[:1] == b"\x8F"
+                        and reply_data[:1] == b"\x8f"
                         and reply_data[1:3] == request_data[:2]
                     ):
                         error = ord(reply_data[3:4])
@@ -485,10 +583,10 @@ def request(handle, devnumber, request_id, *params, no_reply=False, return_error
                                 devnumber,
                                 request_id,
                                 error,
-                                _hidpp10_constants.ERROR[error],
+                                Hidpp10ErrorCode(error),
                             )
-                        return _hidpp10_constants.ERROR[error] if return_error else None
-                    if reply_data[:1] == b"\xFF" and reply_data[1:3] == request_data[:2]:
+                        return Hidpp10ErrorCode(error) if return_error else None
+                    if reply_data[:1] == b"\xff" and reply_data[1:3] == request_data[:2]:
                         # a HID++ 2.0 feature call returned with an error
                         error = ord(reply_data[3:4])
                         logger.error(
@@ -497,9 +595,14 @@ def request(handle, devnumber, request_id, *params, no_reply=False, return_error
                             devnumber,
                             request_id,
                             error,
-                            _hidpp20_constants.ERROR[error],
+                            Hidpp20ErrorCode(error),
+                        )
+                        raise exceptions.FeatureCallError(
+                            number=devnumber,
+                            request=request_id,
+                            error=error,
+                            params=params,
                         )
-                        raise exceptions.FeatureCallError(number=devnumber, request=request_id, error=error, params=params)
 
                     if reply_data[:2] == request_data[:2]:
                         if devnumber == 0xFF:
@@ -517,20 +620,13 @@ def request(handle, devnumber, request_id, *params, no_reply=False, return_error
                 else:
                     # a reply was received, but did not match our request in any way
                     # reset the timeout starting point
-                    request_started = _timestamp()
+                    request_started = time()
 
                 if notifications_hook:
                     n = make_notification(report_id, reply_devnumber, reply_data)
                     if n:
                         notifications_hook(n)
-                    # elif logger.isEnabledFor(logging.DEBUG):
-                    #     logger.debug("(%s) ignoring reply %02X [%s]", handle, reply_devnumber, _strhex(reply_data))
-                # elif logger.isEnabledFor(logging.DEBUG):
-                #     logger.debug("(%s) ignoring reply %02X [%s]", handle, reply_devnumber, _strhex(reply_data))
-
-            delta = _timestamp() - request_started
-            # if logger.isEnabledFor(logging.DEBUG):
-            #     logger.debug("(%s) still waiting for reply, delta %f", handle, delta)
+            delta = time() - request_started
 
         logger.warning(
             "timeout (%0.2f/%0.2f) on device %d request {%04X} params [%s]",
@@ -538,12 +634,12 @@ def request(handle, devnumber, request_id, *params, no_reply=False, return_error
             timeout,
             devnumber,
             request_id,
-            _strhex(params),
+            common.strhex(params),
         )
         # raise DeviceUnreachable(number=devnumber, request=request_id)
 
 
-def ping(handle, devnumber, long_message=False):
+def ping(handle, devnumber, long_message: bool = False):
     """Check if a device is connected to the receiver.
     :returns: The HID protocol supported by the device, as a floating point number, if the device is active.
     """
@@ -557,14 +653,13 @@ def ping(handle, devnumber, long_message=False):
             logger.warning("device or receiver disconnected")
             return
 
-        # randomize the SoftwareId and mark byte to be able to identify the ping
-        # reply, and set most significant (0x8) bit in SoftwareId so that the reply
-        # is always distinguishable from notifications
-        request_id = 0x0018 | _random_bits(3)
-        request_data = _pack("!HBBB", request_id, 0, 0, _random_bits(8))
+        # randomize the mark byte to be able to identify the ping reply
+        sw_id = _get_next_sw_id()
+        request_id = 0x0010 | sw_id  # was 0x0018 | getrandbits(3)
+        request_data = struct.pack("!HBBB", request_id, 0, 0, getrandbits(8))
         write(int(handle), devnumber, request_data, long_message)
 
-        request_started = _timestamp()  # we consider timeout from this point
+        request_started = time()  # we consider timeout from this point
         delta = 0
         while delta < _PING_TIMEOUT:
             reply = _read(handle, _PING_TIMEOUT)
@@ -577,18 +672,16 @@ def ping(handle, devnumber, long_message=False):
 
                     if (
                         report_id == HIDPP_SHORT_MESSAGE_ID
-                        and reply_data[:1] == b"\x8F"
+                        and reply_data[:1] == b"\x8f"
                         and reply_data[1:3] == request_data[:2]
                     ):  # error response
                         error = ord(reply_data[3:4])
-                        if error == _hidpp10_constants.ERROR.invalid_SubID__command:  # a valid reply from a HID++ 1.0 device
+                        if error == Hidpp10ErrorCode.INVALID_SUB_ID_COMMAND:
+                            # a valid reply from a HID++ 1.0 device
                             return 1.0
-                        if (
-                            error == _hidpp10_constants.ERROR.resource_error
-                            or error == _hidpp10_constants.ERROR.connection_request_failed
-                        ):
+                        if error in [Hidpp10ErrorCode.RESOURCE_ERROR, Hidpp10ErrorCode.CONNECTION_REQUEST_FAILED]:
                             return  # device unreachable
-                        if error == _hidpp10_constants.ERROR.unknown_device:  # no paired device with that number
+                        if error == Hidpp10ErrorCode.UNKNOWN_DEVICE:  # no paired device with that number
                             logger.error("(%s) device %d error on ping request: unknown device", handle, devnumber)
                             raise exceptions.NoSuchDevice(number=devnumber, request=request_id)
 
@@ -596,9 +689,7 @@ def ping(handle, devnumber, long_message=False):
                     n = make_notification(report_id, reply_devnumber, reply_data)
                     if n:
                         notifications_hook(n)
-                    # elif logger.isEnabledFor(logging.DEBUG):
-                    #     logger.debug("(%s) ignoring reply %02X [%s]", handle, reply_devnumber, _strhex(reply_data))
 
-            delta = _timestamp() - request_started
+            delta = time() - request_started
 
         logger.warning("(%s) timeout (%0.2f/%0.2f) on device %d ping", handle, delta, _PING_TIMEOUT, devnumber)
